/**
 * This program has been developed by students from the bachelor Computer Science at
 * Utrecht University within the Software Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

/* istanbul ignore file */
/* The comment above was added so the code coverage wouldn't count this file towards code coverage.
 * We do not test components/renderfunctions/styling files.
 * See testing plan for more details.*/
import {
  setQuerybuilderNodes,
  useAppDispatch,
  useQuerybuilderGraph,
  useQuerybuilderGraphology,
  useQuerybuilderHash,
} from '@graphpolaris/shared/lib/data-access';
import { useCallback, useEffect, useMemo, useState } from 'react';
import { Handle, HandleType, Position } from 'reactflow';
import { LogicNodeAttributes, SchemaReactflowLogicNode, toHandleId } from '../../../model';
import { InputNode, InputNodeTypeTypes } from '../../../model/logic/general';
import { styleHandleMap } from '../../utils';
import styles from './logicpill.module.scss';

/**
 * Component to render an entity flow element
 * @param param0 Data of the flow element.
 */
export default function LogicPill(node: SchemaReactflowLogicNode) {
  const dispatch = useAppDispatch();
  const data = node.data;
  const logic = data.logic;
  const output = data.logic.output;

  const graph = useQuerybuilderGraph();
  const graphology = useQuerybuilderGraphology();
  const graphologyHash = useQuerybuilderHash();
  const connectionsToLeft = useMemo(() => graph.edges.filter((edge) => edge.target === node.id), [graph]);
  const connectionsToRight = useMemo(() => graph.edges.filter((edge) => edge.source === node.id), [graph]);
  const graphologyNodeAttributes = useMemo<LogicNodeAttributes | undefined>(
    () => (graphology.hasNode(node.id) ? { ...(graphology.getNodeAttributes(node.id) as LogicNodeAttributes) } : undefined),
    [node.id]
  );
  const [localInputCache, setLocalInputCache] = useState<Record<string, InputNodeTypeTypes>>({ ...graphologyNodeAttributes?.inputs });

  if (!data.id) throw new Error('LogicPill: data.id is undefined');
  const defaultHandleData = {
    nodeId: data.id,
    nodeName: data.name,
    nodeType: data.type,
  };

  const onInputUpdated = (value: string, input: InputNode, idx: number) => {
    let logicNode = { ...graphologyNodeAttributes };
    if (!logicNode) throw new Error('LogicPill: logicNode is undefined');

    let logicNodeInputs = { ...logicNode.inputs };
    if (logicNodeInputs[input.name] != value) {
      logicNodeInputs[input.name] = value;
      logicNode.inputs = logicNodeInputs;
      graphology.setNodeAttribute<any>(node.id, 'inputs', logicNodeInputs); // FIXME: I'm not sure why TS requires <any> to work here
      dispatch(setQuerybuilderNodes(graphology.export()));
      console.log('updated input', input.name, 'to', value);
    }
  };

  const createLeftHandles = useCallback(
    (sideInputs: InputNode[]) => {
      return sideInputs.filter((input, i) => {
        return !connectionsToLeft.some(
          (edge) => edge?.attributes?.targetHandleData.nodeId === data.id && edge?.attributes?.targetHandleData.attributeName === input.name
        );
      }).length;
    },
    [node]
  );
  const leftInputsNumber = createLeftHandles(node.data.logic.inputs);

  return (
    <div className={styles.logic}>
      <span className={styles.logicSpan} style={{ height: `${Math.max(leftInputsNumber * 2.6, 1.5)}rem` }}>
        {
          <span>
            {connectionsToLeft.map((e) => e?.attributes?.sourceHandleData.attributeName)}.{output.name}
          </span>
        }
        {node.data.logic.inputs.map((input, i) => {
          let inputTextBox = null;
          if (
            !connectionsToLeft.some(
              (edge) =>
                edge?.attributes?.targetHandleData.nodeId === data.id && edge?.attributes?.targetHandleData.attributeName === input.name
            )
          ) {
            inputTextBox = (
              <input
                className={styles.logicInput}
                value={localInputCache?.[input.name] as string}
                style={{ top: -5, transform: `translateY(-${i * 20}%)` }}
                onChange={(e) => {
                  setLocalInputCache({ ...localInputCache, [input.name]: e.target.value });
                }}
                onKeyDown={(e) => {
                  if (e.key === 'Enter') onInputUpdated((e.target as HTMLInputElement).value, input, i);
                }}
                onBlur={(e) => onInputUpdated(e.target.value, input, i)}
              />
            );
          }
          return (
            <Handle
              type={'target'}
              position={Position.Left}
              id={toHandleId({ ...defaultHandleData, attributeName: input.name, attributeType: input.type })}
              key={input.name + input.type}
              style={{ top: `${((i + 0.8) / (node.data.logic.inputs.length + 0.6)) * 100}%` }}
              className={styleHandleMap[input.type]}
            >
              {inputTextBox}
            </Handle>
          );
        })}
        {!!node.data.logic.output && (
          <Handle
            type={'source'}
            position={Position.Right}
            id={toHandleId({ ...defaultHandleData, attributeName: output.name, attributeType: output.type })}
            className={styleHandleMap?.[output.type]}
          ></Handle>
        )}
      </span>
    </div>
  );
}
